/***********************************************************************
**                                                                    **
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    **
***********************************************************************/
#include <math.h>
#include "struct.h"

#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4
/* The value ERRCON equals (5/SAFETY) raised to the power (1/PGROW), see use below. */
#define ZERO 0.0

static float maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
(maxarg1) : (maxarg2))
static float minarg1,minarg2;
#define FMIN(a,b) (minarg1=(a),minarg2=(b),(minarg1) < (minarg2) ?\
(minarg1) : (minarg2))

extern void rkck(real *y, real *dydx, int n, real x, real h,real *yout, real *yerr);
extern void shrunk(real,real,long,real,real,char*);
extern void warn_negative(real,long,real,char*);

extern real *ytemp,*yerr;

void rkqs(int meth,real *y, real *dydx, int n, real *x, real htry, real __acc,real __mindelt,real *yscal, real *hdid, real *hnext)
/*Fifth-order Runge-Kutta step with monitoring of local truncation error to ensure accuracy and
adjust stepsize. Input are the dependent variable vector y[1..n] and its derivative dydx[1..n]
at the starting value of the independent variable x. Also input are the stepsize to be attempted
htry, the required accuracy __acc, and the vector yscal[1..n] against which the error is
scaled. On output, y and x are replaced by their new values, hdid is the stepsize that was
actuallyac complished, and hnext is the estimated next stepsize. */
{
        int i,nega,__accept_nega,shr_delt=0;
        real errmax,h,htemp,hshrunk=0.0,xnew,temp;
        string error;    
    
        h=htry; /*Set stepsize to the initial trial value.*/
        if (fabs(h)<__mindelt) 
        {
                sprintf(error,"Step size too small in RKQS. delt=%g",(double) h);
                std_error(error);
        }

        for (;;) {
                rkck(y,dydx,n,*x,h,ytemp,yerr); /*Take a step.*/
                errmax=0.0; /*Evaluate accuracy.*/
                nega=0;
                __accept_nega=0;
                 for (i=1;i<=n;i++) 
                {
#if defined (__accept_nega_with_zero_rate__)
                    if (y[i] < ZERO && ytemp[i] == y[i]) 
                   {
						__accept_nega=1;
#if defined (__verbose__)
 					   warn_negative(*x,i,y[i],"rkqs");
#endif
				    }
#endif	
                    if (meth>0 && __accept_nega!=1 && (y[i] < ZERO && ytemp[i] ==y[i]))
                    {
                       nega=1;
                       hshrunk=h;
                       shr_delt=i;
                       break;
                    }
                    temp=fabs(yerr[i]/yscal[i]);
                    if (errmax<temp)
                    {
                        errmax=temp;
                        hshrunk=h;
                        shr_delt=i;
                    }
                }
                if (__acc > 0.0)
                  {
                    errmax /= __acc; /*Scale relative to required tolerance.*/
                  }
                  else
                  {
                    printf("Warning: Accuracy was set to 0 for Cash-Karp Integrator. No adaptation computed.\n");
		    fflush(stdout);  
		  }

                if (errmax <= 1.0 && nega==0)
                {
                      if (errmax > ERRCON) *hnext=SAFETY*h*pow(errmax,PGROW);
                      else *hnext=5.0*h; /*No more than a factor of 5 increase.*/
                      break; /*Step succeeded. Compute size of next step.*/
                }
                if (nega==1) htemp=h/2.0; else htemp=SAFETY*h*pow(errmax,PSHRNK);
                     /* Truncation error too large, reduce stepsize. */
                    h=(h >= 0.0 ? FMAX(htemp,0.1*h) : FMIN(htemp,0.1*h));
                /* No more than a factor of 10. */
                shrunk(*x,hshrunk,shr_delt,ytemp[shr_delt],-(ytemp[shr_delt]-y[shr_delt])/hshrunk,"rkck");
                xnew=(*x)+h;
                if (xnew <= *x) std_error("stepsize underflow in rkqs");
        }
        *x += (*hdid=h);
        for (i=1;i<=n;i++) y[i]=ytemp[i];
}
