/***********************************************************************
**                                                                    **
** CEMoS Version 4.1 Copyright (C) 2012, Frank Hamberg, Cora Kohlmeier**
**                                                                    **
** last change 2012/08/25 -> removed numerical recipes relations                 **
**                                                                    **
***********************************************************************/

#include "stdinc.h"
#include "types.h"
#define C_OFFSET 1
#define FREE_ARG char*

void std_error(const char error_text[])
{
	printf("CeMoS message: %s\n",error_text);
	printf("...now exiting to the system...\n");
        fflush(stdout);
	exit(EXIT_FAILURE);
}

real *vector(long first, long last)
/* allocate a real vector with subscript range v[first..last] */
{
	real *v;
        long i;

	v=(real *) malloc ((size_t) ((last-first+1+C_OFFSET)*sizeof(real)));
	if (!v) std_error("allocation failure in vector()");
        for (i=first+1;i<=last;i++) v[i]=0.0;
	return v-first+C_OFFSET;
}

float *fvector(long first, long last)
/* allocate a float vector with subscript range v[first..last] */
{
	float *v;
        long i;

	v=(float *) malloc ((size_t) ((last-first+1+C_OFFSET)*sizeof(float)));
	if (!v) std_error("allocation failure in fvector()");
        for (i=first+1;i<=last;i++) v[i]=0.0;
	return v-first+C_OFFSET;
}

int *ivector(long first, long last)
/* allocate an int vector with subscript range v[first..last] */
{
	int *v;
        long i;

	v=(int *) malloc ((size_t) ((last-first+1+C_OFFSET)*sizeof(int)));
	if (!v) std_error("allocation failure in ivector()");
        for (i=first+1;i<=last;i++) v[i]=0;
	return v-first+C_OFFSET;
}

unsigned char *cvector(long first, long last)
/* allocate an unsigned char vector with subscript range v[first..last] */
{
	unsigned char *v;
        long i;

	v=(unsigned char *) malloc ((size_t) ((last-first+1+C_OFFSET)*sizeof(unsigned char)));
	if (!v) std_error("allocation failure in cvector()");
        for (i=first+1;i<=last;i++) v[i]=(char) 0;
	return v-first+C_OFFSET;
}

unsigned long *lvector(long first, long last)
/* allocate an unsigned long vector with subscript range v[first..last] */
{
	unsigned long *v;
        long i;

	v=(unsigned long *) malloc ((size_t) ((last-first+1+C_OFFSET)*sizeof(long)));
	if (!v) std_error("allocation failure in lvector()");
        for (i=first+1;i<=last;i++) v[i]=0;
	return v-first+C_OFFSET;
}

double *dvector(long first, long last)
/* allocate a double vector with subscript range v[first..last] */
{
	double *v;
        long i;

	v=(double *) malloc ((size_t) ((last-first+1+C_OFFSET)*sizeof(double)));
	if (!v) std_error("allocation failure in dvector()");
        for (i=first+1;i<=last;i++) v[i]=0.0;
	return v-first+C_OFFSET;
}

real **matrix(long first_row, long last_row, long first_col, long last_col)
/* allocate a real matrix with subscript range m[first_row..last_row][first_col..last_col] */
{
	long i, num_rows=last_row-first_row+1,num_cols=last_col-first_col+1;
	real **m;

	/* allocate pointers to rows */
	m=(real **) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(real*)));
	if (!m) std_error("allocation failure 1 in matrix()");
	m += C_OFFSET;
	m -= first_row;

	/* allocate rows and set pointers to them */
	m[first_row]=(real *) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(real)));
	if (!m[first_row]) std_error("allocation failure 2 in matrix()");
	m[first_row] += C_OFFSET;
	m[first_row] -= first_col;

	for(i=first_row+1;i<=last_row;i++) m[i]=m[i-1]+num_cols;

	/* return pointer to array of pointers to rows */
	return m;
}

float **fmatrix(long first_row, long last_row, long first_col, long last_col)
/* allocate a float matrix with subscript range m[first_row..last_row][first_col..last_col] */
{
	long i, num_rows=last_row-first_row+1,num_cols=last_col-first_col+1;
	float **m;

	/* allocate pointers to rows */
	m=(float **) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(float*)));
	if (!m) std_error("allocation failure 1 in fmatrix()");
	m += C_OFFSET;
	m -= first_row;

	/* allocate rows and set pointers to them */
	m[first_row]=(float *) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(float)));
	if (!m[first_row]) std_error("allocation failure 2 in fmatrix()");
	m[first_row] += C_OFFSET;
	m[first_row] -= first_col;

	for(i=first_row+1;i<=last_row;i++) m[i]=m[i-1]+num_cols;

	/* return pointer to array of pointers to rows */
	return m;
}

double **dmatrix(long first_row, long last_row, long first_col, long last_col)
/* allocate a double matrix with subscript range m[first_row..last_row][first_col..last_col] */
{
	long i, num_rows=last_row-first_row+1,num_cols=last_col-first_col+1;
	double **m;

	/* allocate pointers to rows */
	m=(double **) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(double*)));
	if (!m) std_error("allocation failure 1 in dmatrix()");
	m += C_OFFSET;
	m -= first_row;

	/* allocate rows and set pointers to them */
	m[first_row]=(double *) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(double)));
	if (!m[first_row]) std_error("allocation failure 2 in dmatrix()");
	m[first_row] += C_OFFSET;
	m[first_row] -= first_col;

	for(i=first_row+1;i<=last_row;i++) m[i]=m[i-1]+num_cols;

	/* return pointer to array of pointers to rows */
	return m;
}

int **imatrix(long first_row, long last_row, long first_col, long last_col)
/* allocate a int matrix with subscript range m[first_row..last_row][first_col..last_col] */
{
	long i, num_rows=last_row-first_row+1,num_cols=last_col-first_col+1;
	int **m;

	/* allocate pointers to rows */
	m=(int **) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(int*)));
	if (!m) std_error("allocation failure 1 in imatrix()");
	m += C_OFFSET;
	m -= first_row;


	/* allocate rows and set pointers to them */
	m[first_row]=(int *) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(int)));
	if (!m[first_row]) std_error("allocation failure 2 in imatrix()");
	m[first_row] += C_OFFSET;
	m[first_row] -= first_col;

	for(i=first_row+1;i<=last_row;i++) m[i]=m[i-1]+num_cols;

	/* return pointer to array of pointers to rows */
	return m;
}

long int **lmatrix(long first_row, long last_row, long first_col, long last_col)
/* allocate a int matrix with subscript range m[first_row..last_row][first_col..last_col] */
{
	long i, num_rows=last_row-first_row+1,num_cols=last_col-first_col+1;
	long int **m;

	/* allocate pointers to rows */
	m=(long int **) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(long int*)));
	if (!m) std_error("allocation failure 1 in lmatrix()");
	m += C_OFFSET;
	m -= first_row;


	/* allocate rows and set pointers to them */
	m[first_row]=(long int *) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(long int)));
	if (!m[first_row]) std_error("allocation failure 2 in lmatrix()");
	m[first_row] += C_OFFSET;
	m[first_row] -= first_col;

	for(i=first_row+1;i<=last_row;i++) m[i]=m[i-1]+num_cols;

	/* return pointer to array of pointers to rows */
	return m;
}

float **submatrix(float **in_ptr, long old_first_row, long old_last_row, long old_first_col, long old_last_col,
	long new_first_row, long new_first_col)
/* point a submatrix [new_first_row..][new_first_col..] to in_ptr[old_first_row..old_last_row][old_first_col..old_last_col] */
{
	long i,j,num_rows=old_last_row-old_first_row+1,num_cols=old_first_col-new_first_col;
	float **m;
        i=old_last_col; /*dummy*/
	/* allocate array of pointers to rows */
	m=(float **) malloc ((size_t) ((num_rows+C_OFFSET)*sizeof(float*)));
	if (!m) std_error("allocation failure in submatrix()");
	m += C_OFFSET;
	m -= new_first_row;

	/* set pointers to rows */
	for(i=old_first_row,j=new_first_row;i<=old_last_row;i++,j++) m[j]=in_ptr[i]+num_cols;

	/* return pointer to array of pointers to rows */
	return m;
}

float **convert_matrix(float *in_ptr, long first_row, long last_row, long first_col, long last_col)
/* allocate a float matrix m[first_row..last_row][first_col..last_col] that points to the matrix
declared in the standard C manner as in_ptr[num_rows][num_cols], where num_rows=last_row-first_row+1
and num_cols=last_col-first_col+1. The routine should be called with the address
&in_ptr[0][0] as the first argument. */
{
	long i,j,num_rows=last_row-first_row+1,num_cols=last_col-first_col+1;
	float **m;

	/* allocate pointers to rows */
	m=(float **) malloc ((size_t) ((num_rows+C_OFFSET)*sizeof(float*)));
	if (!m) std_error("allocation failure in convert_matrix()");
	m += C_OFFSET;
	m -= first_row;

	/* set pointers to rows */
	m[first_row]=in_ptr-first_col;
	for(i=1,j=first_row+1;i<num_rows;i++,j++) m[j]=m[j-1]+num_cols;
	/* return pointer to array of pointers to rows */
	return m;
}

float ***f3tensor(long first_row, long last_row, long first_col, long last_col, long first_lay, long last_lay)
/* allocate a float 3tensor with range t[first_row..last_row][first_col..last_col][first_lay..last_lay] */
{
	long i,j,num_rows=last_row-first_row+1,num_cols=last_col-first_col+1,ndep=last_lay-first_lay+1;
	float ***t;

	/* allocate pointers to pointers to rows */
	t=(float ***) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(float**)));
	if (!t) std_error("allocation failure 1 in f3tensor()");
	t += C_OFFSET;
	t -= first_row;

	/* allocate pointers to rows and set pointers to them */
	t[first_row]=(float **) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(float*)));
	if (!t[first_row]) std_error("allocation failure 2 in f3tensor()");
	t[first_row] += C_OFFSET;
	t[first_row] -= first_col;

	/* allocate rows and set pointers to them */
	t[first_row][first_col]=(float *) malloc ((size_t)((num_rows*num_cols*ndep+C_OFFSET)*sizeof(float)));
	if (!t[first_row][first_col]) std_error("allocation failure 3 in f3tensor()");
	t[first_row][first_col] += C_OFFSET;
	t[first_row][first_col] -= first_lay;

	for(j=first_col+1;j<=last_col;j++) t[first_row][j]=t[first_row][j-1]+ndep;
	for(i=first_row+1;i<=last_row;i++) {
		t[i]=t[i-1]+num_cols;
		t[i][first_col]=t[i-1][first_col]+num_cols*ndep;
		for(j=first_col+1;j<=last_col;j++) t[i][j]=t[i][j-1]+ndep;
	}

	/* return pointer to array of pointers to rows */
	return t;
}

double ***d3tensor(long first_row, long last_row, long first_col, long last_col, long first_lay, long last_lay)
/* allocate a float 3tensor with range t[first_row..last_row][first_col..last_col][first_lay..last_lay] */
{
	long i,j,num_rows=last_row-first_row+1,num_cols=last_col-first_col+1,ndep=last_lay-first_lay+1;
	double ***t;

	/* allocate pointers to pointers to rows */
	t=(double ***) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(double**)));
	if (!t) std_error("allocation failure 1 in d3tensor()");
	t += C_OFFSET;
	t -= first_row;

	/* allocate pointers to rows and set pointers to them */
	t[first_row]=(double **) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(double*)));
	if (!t[first_row]) std_error("allocation failure 2 in d3tensor()");
	t[first_row] += C_OFFSET;
	t[first_row] -= first_col;

	/* allocate rows and set pointers to them */
	t[first_row][first_col]=(double *) malloc ((size_t)((num_rows*num_cols*ndep+C_OFFSET)*sizeof(double)));
	if (!t[first_row][first_col]) std_error("allocation failure 3 in d3tensor()");
	t[first_row][first_col] += C_OFFSET;
	t[first_row][first_col] -= first_lay;

	for(j=first_col+1;j<=last_col;j++) t[first_row][j]=t[first_row][j-1]+ndep;
	for(i=first_row+1;i<=last_row;i++) {
		t[i]=t[i-1]+num_cols;
		t[i][first_col]=t[i-1][first_col]+num_cols*ndep;
		for(j=first_col+1;j<=last_col;j++) t[i][j]=t[i][j-1]+ndep;
	}

	/* return pointer to array of pointers to rows */
	return t;
}

int ***i3tensor(long first_row, long last_row, long first_col, long last_col, long first_lay, long last_lay)
/* allocate an integer 3tensor with range t[first_row..last_row][first_col..last_col][first_lay..last_lay] */
{
	long i,j,num_rows=last_row-first_row+1,num_cols=last_col-first_col+1,ndep=last_lay-first_lay+1;
	int ***t;

	/* allocate pointers to pointers to rows */
	t=(int ***) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(int**)));
	if (!t) std_error("allocation failure 1 in i3tensor()");
	t += C_OFFSET;
	t -= first_row;

	/* allocate pointers to rows and set pointers to them */
	t[first_row]=(int **) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(int*)));
	if (!t[first_row]) std_error("allocation failure 2 in i3tensor()");
	t[first_row] += C_OFFSET;
	t[first_row] -= first_col;

	/* allocate rows and set pointers to them */
	t[first_row][first_col]=(int *) malloc ((size_t)((num_rows*num_cols*ndep+C_OFFSET)*sizeof(int)));
	if (!t[first_row][first_col]) std_error("allocation failure 3 in i3tensor()");
	t[first_row][first_col] += C_OFFSET;
	t[first_row][first_col] -= first_lay;

	for(j=first_col+1;j<=last_col;j++) t[first_row][j]=t[first_row][j-1]+ndep;
	for(i=first_row+1;i<=last_row;i++) {
		t[i]=t[i-1]+num_cols;
		t[i][first_col]=t[i-1][first_col]+num_cols*ndep;
		for(j=first_col+1;j<=last_col;j++) t[i][j]=t[i][j-1]+ndep;
	}

	/* return pointer to array of pointers to rows */
	return t;
}

real ***r3tensor(long first_row, long last_row, long first_col, long last_col, long first_lay, long last_lay)
/* allocate a real 3tensor with range t[first_row..last_row][first_col..last_col][first_lay..last_lay] */
{
	long i,j,num_rows=last_row-first_row+1,num_cols=last_col-first_col+1,ndep=last_lay-first_lay+1;
	real ***t;

	/* allocate pointers to pointers to rows */
	t=(real ***) malloc ((size_t)((num_rows+C_OFFSET)*sizeof(real**)));
	if (!t) std_error("allocation failure 1 in r3tensor()");
	t += C_OFFSET;
	t -= first_row;

	/* allocate pointers to rows and set pointers to them */
	t[first_row]=(real **) malloc ((size_t)((num_rows*num_cols+C_OFFSET)*sizeof(real*)));
	if (!t[first_row]) std_error("allocation failure 2 in r3tensor()");
	t[first_row] += C_OFFSET;
	t[first_row] -= first_col;

	/* allocate rows and set pointers to them */
	t[first_row][first_col]=(real *) malloc ((size_t)((num_rows*num_cols*ndep+C_OFFSET)*sizeof(real)));
	if (!t[first_row][first_col]) std_error("allocation failure 3 in r3tensor()");
	t[first_row][first_col] += C_OFFSET;
	t[first_row][first_col] -= first_lay;

	for(j=first_col+1;j<=last_col;j++) t[first_row][j]=t[first_row][j-1]+ndep;
	for(i=first_row+1;i<=last_row;i++) {
		t[i]=t[i-1]+num_cols;
		t[i][first_col]=t[i-1][first_col]+num_cols*ndep;
		for(j=first_col+1;j<=last_col;j++) t[i][j]=t[i][j-1]+ndep;
	}

	/* return pointer to array of pointers to rows */
	return t;
}
real **r3tensor_submatrix(real ***in_ptr, long newr, long first_col, long last_col, long first_lay, long last_lay)
/* point a submatrix [newr] to tensor in_ptr[newr][first_col..last_col][first_lay..last_lay] */
{
        long i,num_cols=last_col-first_col+1;
        real **m;
        i=last_lay; /*dummy for lint*/
        /* allocate array of pointers to rows */
        m=(real **) malloc ((size_t) ((num_cols+C_OFFSET)*sizeof(real*)));
        if (!m) std_error("allocation failure in r3tensor_submatrix()");
/*         m += C_OFFSET; */
 
        /* set pointers to rows */
        for(i=first_col;i<=last_col;i++) m[i]=(real*) (&in_ptr[newr][i][first_lay-C_OFFSET]);

        /* return pointer to array of pointers to rows */
        return m;
}

int **i3tensor_submatrix(int ***in_ptr, long newr, long first_col, long last_col, long first_lay, long last_lay)
/* point a submatrix [newr] to tensor in_ptr[newr][first_col..last_col][first_lay..last_lay] */
{
        long i,num_cols=last_col-first_col+1;
        int **m;
        i=last_lay; /*dummy for lint*/

        /* allocate array of pointers to rows */
        m=(int **) malloc ((size_t) ((num_cols+C_OFFSET)*sizeof(int*)));
        if (!m) std_error("allocation failure in i3tensor_submatrix()");
/*         m += C_OFFSET; */
 
        /* set pointers to rows */
        for(i=first_col;i<=last_col;i++) m[i]=(int*) (&in_ptr[newr][i][first_lay-C_OFFSET]);

        /* return pointer to array of pointers to rows */
        return m;
}

void free_vector(real *v, long first, long last)
/* free a float vector allocated with vector() */
{
        long i  __attribute__((unused));
        i=last; /*dummy for lint*/
        free((FREE_ARG) (v+first-C_OFFSET));
}

void free_fvector(float *v, long first, long last)
/* free a float vector allocated with vector() */
{
        long i  __attribute__((unused));
        i=last; /*dummy for lint*/
        free((FREE_ARG) (v+first-C_OFFSET));
}

void free_ivector(int *v, long first, long last)
/* free an int vector allocated with ivector() */
{
         long i  __attribute__((unused));
        i=last; /*dummy for lint*/
	free((FREE_ARG) (v+first-C_OFFSET));
}

void free_cvector(unsigned char *v, long first, long last)
/* free an unsigned char vector allocated with cvector() */
{
        long i  __attribute__((unused));
        i=last; /*dummy for lint*/
	free((FREE_ARG) (v+first-C_OFFSET));
}

void free_lvector(unsigned long *v, long first, long last)
/* free an unsigned long vector allocated with lvector() */
{
        long i  __attribute__((unused));
        i=last; /*dummy for lint*/
	free((FREE_ARG) (v+first-C_OFFSET));
}

void free_dvector(double *v, long first, long last)
/* free a double vector allocated with dvector() */
{
        long i  __attribute__((unused));
        i=last; /*dummy for lint*/
	free((FREE_ARG) (v+first-C_OFFSET));
}

void free_matrix(real **m, long first_row, long last_row, long first_col, long last_col)
/* free a float matrix allocated by matrix() */
{
        long i  __attribute__((unused));
        i =last_row; /*dummy for lint*/
        i=last_col; /*dummy for lint*/

        free((FREE_ARG) (m[first_row]+first_col-C_OFFSET));
        free((FREE_ARG) (m+first_row-C_OFFSET));
}

void free_fmatrix(float **m, long first_row, long last_row, long first_col, long last_col)
/* free a float matrix allocated by matrix() */
{
        long i  __attribute__((unused));
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
        free((FREE_ARG) (m[first_row]+first_col-C_OFFSET));
        free((FREE_ARG) (m+first_row-C_OFFSET));
}

void free_dmatrix(double **m, long first_row, long last_row, long first_col, long last_col)
/* free a double matrix allocated by dmatrix() */
{
        long i  __attribute__((unused));
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (m[first_row]+first_col-C_OFFSET));
	free((FREE_ARG) (m+first_row-C_OFFSET));
}

void free_imatrix(int **m, long first_row, long last_row, long first_col, long last_col)
/* free an int matrix allocated by imatrix() */
{
        long i  __attribute__((unused));
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (m[first_row]+first_col-C_OFFSET));
	free((FREE_ARG) (m+first_row-C_OFFSET));
}

void free_lmatrix(long int **m, long first_row, long last_row, long first_col, long last_col)
/* free an int matrix allocated by imatrix() */
{
        long i  __attribute__((unused));
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (m[first_row]+first_col-C_OFFSET));
	free((FREE_ARG) (m+first_row-C_OFFSET));
}

void free_submatrix(float **b, long first_row, long last_row, long first_col, long last_col)
/* free a submatrix allocated by submatrix() */
{
        long i  __attribute__((unused));
        i=first_col; /*dummy for lint*/
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (b+first_row-C_OFFSET));
}

void free_convert_matrix(float **b, long first_row, long last_row, long first_col, long last_col)
/* free a matrix allocated by convert_matrix() */
{
        long i  __attribute__((unused));
        i=first_col; /*dummy for lint*/
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (b+first_row-C_OFFSET));
}

void free_f3tensor(float ***t, long first_row, long last_row, long first_col, long last_col,
	long first_lay, long last_lay)
/* free a float f3tensor allocated by f3tensor() */
{
        long i  __attribute__((unused));
        i=last_lay; /*dummy for lint*/
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (t[first_row][first_col]+first_lay-C_OFFSET));
	free((FREE_ARG) (t[first_row]+first_col-C_OFFSET));
	free((FREE_ARG) (t+first_row-C_OFFSET));
}

void free_d3tensor(double ***t, long first_row, long last_row, long first_col, long last_col,
	long first_lay, long last_lay)
/* free a double d3tensor allocated by f3tensor() */
{
        long i  __attribute__((unused));
        i=last_lay; /*dummy for lint*/
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (t[first_row][first_col]+first_lay-C_OFFSET));
	free((FREE_ARG) (t[first_row]+first_col-C_OFFSET));
	free((FREE_ARG) (t+first_row-C_OFFSET));
}

void free_i3tensor(int ***t, long first_row, long last_row, long first_col, long last_col,
	long first_lay, long last_lay)
/* free a float i3tensor allocated by f3tensor() */
{
        long i  __attribute__((unused));
        i=last_lay; /*dummy for lint*/
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (t[first_row][first_col]+first_lay-C_OFFSET));
	free((FREE_ARG) (t[first_row]+first_col-C_OFFSET));
	free((FREE_ARG) (t+first_row-C_OFFSET));
}

void free_r3tensor(real ***t, long first_row, long last_row, long first_col, long last_col,
	long first_lay, long last_lay)
/* free a real f3tensor allocated by r3tensor() */
{
        long i  __attribute__((unused));
        i=last_lay; /*dummy for lint*/
        i=last_col; /*dummy for lint*/
        i=last_row; /*dummy for lint*/
	free((FREE_ARG) (t[first_row][first_col]+first_lay-C_OFFSET));
	free((FREE_ARG) (t[first_row]+first_col-C_OFFSET));
	free((FREE_ARG) (t+first_row-C_OFFSET));
}


